@@ -73,29 +73,26 @@ void pqxx::CachedResult::MoveTo(blocknum Block) const
 }
 
 
-pqxx::Result pqxx::CachedResult::Fetch() const
+const pqxx::Result &pqxx::CachedResult::Fetch() const
 {
   size_type Pos = m_Cursor.Pos();
 
   Result R( m_Cursor.Fetch(m_Granularity) );
-  if (R.empty())
+
+  if (!R.empty()) 
   {
-    if (m_HaveEmpty)
-    {
-      R = m_EmptyResult;
-    }
-    else
-    {
-      m_EmptyResult = R;
-      m_HaveEmpty = true;
-    }
-  }
-  else
-  {
-    m_Cache.insert(make_pair(BlockFor(Pos), CacheEntry(R)));
+    m_Cache.insert(make_pair(BlockFor(Pos), R));
+    // TODO: Use iterator returned by insert().  
+    return m_Cache[BlockFor(Pos)];
   }
 
-  return R;
+  if (!m_HaveEmpty)
+  {
+    m_EmptyResult = R;
+    m_HaveEmpty = true;
+  }
+
+  return m_EmptyResult;
 }
 
 
